#! /usr/bin/perl -w

#### LAST UPDATED: Apr, 2003 ####
# unique-hard.pl version 0.1

###########################################################################
#
# This program checks for multiple instance IDs in the SOURCE file which
# has data in the `hard' data format. A `_n' suffix is added to those 
# instance IDs which occur multiple number of times. `n' corresponds to 
# the number of times the instance ID has already occurred in the SOURCE,
# present occurrence included. This procedure helps attain unique instance 
# IDs in the data file. 
# DESTINATION is a copy of SOURCE, barring the above mentioned change.
#
# USAGE :  unique-hard.pl [OPTIONS] DESTINATION SOURCE
#
# If all the instance IDs across a set of files is to be made unique. The
# files may be combined using the cat command and the combined file 
# provided as source to this program.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }

# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# First pass: Store instance IDs which have duplicates
# Read lines from input file
while($ln = <INP>) {

    if($ln =~ /^\s*([^:]+:)(.*)$/) {
        $instid = $1; }
    else { next; }
    
    # Increment counter in hash
    $inst{$instid}++;
    
}
close(INP);

open(INP,"<$inpfl");
# Second pass: Rename instance IDs
while($ln = <INP>) {
    
    if($ln =~ /^\s*([^:]+:)(.*)$/) {
      	$instid = $1; }
    else { next; }
    
    # Check if instance ID has been repeated
    if($inst{$instid} > 1) {
	$ctrInst{$instid}++;
	$cpyinstid = $instid;
	$cpyinstid =~ s/\:\s*$//;
	$ln =~ s/$instid/$cpyinstid\_$ctrInst{$instid}\:/;
    } 
    print OP "$ln";
    
}

# Subroutine to print help message
sub help{
    print "\nUsage: unique-hard.pl [OPTIONS] DESTINATION SOURCE\n\n"; 
    print "This program checks for multiple instance IDs in the SOURCE file which has data in the `hard' data format. A `_n' suffix is added to those instance IDs which occur multiple number of times. `n' corresponds to the number of times the instance ID has already occurred in the SOURCE, present occurrence included. This procedure helps attain unique instance IDs in the data file.  DESTINATION is a copy of SOURCE, barring the above mentioned change.\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n\n";}

sub usage{
    print "Usage: unique-hard.pl [OPTIONS] DESTINATION SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type unique-hard.pl --help for help.\n";}
